!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   rg_option_mod
!> @brief   module for analysis options
!! @authors Norio Takase (NT)
! 
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../../config.h"
#endif

module rg_option_mod

  use rg_option_str_mod
  use fileio_control_mod
  use messages_mod
  use constants_mod

  implicit none
  private

  ! structures
  type, public :: s_opt_info
    integer                  :: nreplica

    real(wp)                 :: min_energy        = 0.0_wp
    real(wp)                 :: min_delta_r       = 0.0_wp
    integer                  :: md_iseed          = 0
    real(wp)                 :: md_thermo_moment  = 0.0_wp
    real(wp)                 :: md_baro_moment(3) = 0.0_wp
  end type s_opt_info

  ! subroutines
  public  :: show_ctrl_option
  public  :: read_ctrl_option
  public  :: setup_option

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    show_ctrl_option
  !> @brief        show control parameters in OPTION section
  !! @authors      NT
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine show_ctrl_option

    write(MsgOut,'(A)') '[RPATH]'
    write(MsgOut,'(A)') 'nreplica       = 16'
    write(MsgOut,'(A)') ' '

    return

  end subroutine show_ctrl_option

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    read_ctrl_option
  !> @brief        read control parameters in OPTION section
  !! @authors      NT
  !! @param[in]    handle   : unit number of control file
  !! @param[out]   opt_info : OPTION section control parameters information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine read_ctrl_option(handle, opt_info)
  
    ! parameters
    character(*),            parameter     :: Section1 = 'RPATH'
    character(*),            parameter     :: Section2 = 'OPTION'

    ! formal argments
    integer,                 intent(in)    :: handle
    type(s_opt_info),        intent(inout) :: opt_info

    ! local variables
    character(100)           :: line


    ! read parameters
    !

    ! 'RPATH' section
    call begin_ctrlfile_section(handle, Section1)

    call read_ctrlfile_integer(handle, Section1, 'nreplica', &
                               opt_info%nreplica)

    call end_ctrlfile_section(handle)

    ! 'OPTION' section
    call begin_ctrlfile_section(handle, Section2)

    call read_ctrlfile_real   (handle, Section2, 'min_energy', &
                               opt_info%min_energy)

    call read_ctrlfile_real   (handle, Section2, 'min_delta_r', &
                               opt_info%min_delta_r)

    call read_ctrlfile_integer(handle, Section2, 'md_iseed' , &
                               opt_info%md_iseed)

    call read_ctrlfile_real   (handle, Section2, 'md_thermo_moment', &
                               opt_info%md_thermo_moment)

    line = ''
    call read_ctrlfile_string  (handle, Section2, 'md_baro_moment', line)

    if (line /= '') &
      read(line,*,err=900,end=900) opt_info%md_baro_moment

    call end_ctrlfile_section(handle)


    ! write parameters to MsgOut
    !
    write(MsgOut,'(A)') 'Read_Ctrl_Option> Parameters of Rpath'
    write(MsgOut,'(A20,I10)') &
         ' nreplica         = ', opt_info%nreplica
    write(MsgOut,'(A)') ' '

    return

900 call error_msg('Read_Ctrl_Option> format error : "md_baro_moment"')

  end subroutine read_ctrl_option

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    setup_option
  !> @brief        setup option information
  !! @authors      NT
  !! @param[in]    opt_info : OPTION section control parameters information
  !! @param[inout] option   : option information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine setup_option(opt_info, option)

    ! formal arguments
    type(s_opt_info),        intent(in)    :: opt_info
    type(s_option),  target, intent(inout) :: option

    ! local variables
    integer                  :: igrp

    write(MsgOut,'(a)') 'Setup_Option> :'
    write(MsgOut,'(a)') ''

    option%nreplica = opt_info%nreplica

    return

  end subroutine setup_option

end module rg_option_mod
